#include <SSD1306.h>
#include <OLEDDisplay.h>
#include "esp_system.h"
#include <Wire.h>
#include <WiFi.h>
#include <WiFiUdp.h>
#include <Adafruit_PWMServoDriver.h>
                    
#define SSID "**********"          // 無線LANアクセスポイントのSSID
#define PASS "**********"          // パスワード
#define R_PORT 9000
#define PULSE_MIN 0  // this is the 'minimum' pulse length count (out of 4096)
#define PULSE_MAX 2400  // this is the 'maximum' pulse length count (out of 4096)
#define BRIGHT 255
#define BUZZER_PIN 25

//音を鳴らす時間
#define BEAT 500
#define TEMPO 150

//音階名と周波数の対応
#define C3 130.813
#define Cs3 138.591
#define Db3 138.591
#define D3 146.832
#define Ds3 155.563
#define Eb3 155.563
#define E3 164.814
#define Es3 174.614
#define F3 174.614
#define Fs3 184.997
#define Gb3 184.997
#define G3 195.998
#define Gs3 207.652
#define Ab3 207.652
#define A3 220.000
#define As3 233,082
#define Bb3 233,082
#define B3 246.942 

#define C4 261.626 
#define Cs4 277.183
#define Db4 277.183
#define D4 293.665
#define Ds4 311.127
#define Eb4 311.127
#define E4 329.628
#define Es4 349.228
#define F4 349.228
#define Fs4 369.994
#define Gb4 369.994
#define G4 391.995
#define Gs4 415.305
#define Ab4 415.305
#define A4 440.000
#define As4 466.164
#define Bb4 466.164
#define B4 493.883

#define C5 523.251
#define Cs5 554.365
#define Db5 554.365
#define D5 587.330
#define Ds5 622.254
#define Eb5 622.254
#define E5 659.255
#define Es5 698.456
#define F5 698.456
#define Fs5 739.989
#define Gb5 739.989
#define G5 783.991
#define Gs5 830.609
#define Ab5 830.609
#define A5 880.000
#define As5 932.328
#define Bb5 932.328
#define B5 987.767
#define C6 1046.502
#define Cs6 1108.731
#define Db6 1108.731
#define D6 1174.659
#define Ds6 1244.508

String keys[20]={"A","M","%","/","7","8","9","*","4","5","6","-","1","2","3","+",".","0","`","="};

int tones[12][17]={
  {C4,D4,E4,F4,G4,A4,B4,C5,D5,E5,F5,G5,A5,B5,C6,D6,0},
  {C4,D4,E4,Fs4,G4,A4,B4,C5,D5,E5,Fs5,G5,A5,B5,C6,D6,0},
  {Cs4,D4,E4,Fs4,G4,A4,B4,Cs5,D5,E5,Fs5,G5,A5,B5,Cs6,D6,0},
  {Cs4,D4,E4,Fs4,Gs4,A4,B4,Cs5,D5,E5,Fs5,Gs5,A5,B5,Cs6,D6,0},
  {Cs4,Ds4,E4,Fs4,Gs4,A4,B4,Cs5,Ds5,E5,Fs5,Gs5,A5,B5,Cs6,Ds6,0},
  {Cs4,Ds4,E4,Fs4,Gs4,As4,B4,Cs5,Ds5,E5,Fs5,Gs5,As5,B5,Cs6,Ds6,0},
  {Cs4,Ds4,Es4,Fs4,Gs4,As4,B4,Cs5,Ds5,Es5,Fs5,Gs5,As5,B5,Cs6,Ds6,0},
  {C4,Db4,Eb4,F4,Gb4,Ab4,Bb4,C5,Db5,Eb5,F5,Gb5,Ab5,Bb5,C6,Db6,0},
  {C4,Db4,Eb4,F4,G4,Ab4,Bb4,C5,Db5,Eb5,F5,G5,Ab5,Bb5,C6,Db6,0},
  {C4,D4,Eb4,F4,G4,Ab4,Bb4,C5,D5,Eb5,F5,G5,Ab5,Bb5,C6,D6,0},
  {C4,D4,Eb4,F4,G4,A4,Bb4,C5,D5,Eb5,F5,G5,A5,Bb5,C6,D6,0},
  {C4,D4,E4,F4,G4,A4,Bb4,C5,D5,E5,F5,G5,A5,Bb5,C6,D6,0}
};

String labels[12][18]={
  {"C4","D4","E4","F4","G4","A4","B4","C5","D5","E5","F5","G5","A5","B5","C6","D6","-","C/Am"},
  {"C4","D4","E4","F#4","G4","A4","B4","C5","D5","E5","F#5","G5","A5","B5","C6","D6","-","G/Em [#1]"},
  {"C#4","D4","E4","F#4","G4","A4","B4","C#5","D5","E5","F#5","G5","A5","B5","C#6","D6","-","D/Bm [#2]"},
  {"C#4","D4","E4","F#4","G#4","A4","B4","C#5","D5","E5","F#5","G#5","A5","B5","C#6","D6","-","A/F#m [#3]"},
  {"C#4","D#4","E4","F#4","G#4","A4","B4","C#5","D#5","E5","F#5","G#5","A5","B5","C#6","D#6","-","E/C#m [#4]"},
  {"C#4","D#4","E4","F#4","G#4","A#4","B4","C#5","D#5","E5","F#5","G#5","A#5","B5","C#6","D#6","-","B/G#m [#5]"},
  {"C#4","D#4","E#4","F#4","G#4","A#4","B4","C#5","D#5","E#5","F#5","G#5","A#5","B5","C#6","D#6","-","F#/D#m [#6]"},
  {"C4","Db4","Eb4","F4","Gb4","Ab4","Bb4","C5","Db5","Eb5","F5","Gb5","Ab5","Bb5","C6","Db6","-","Db/Bbm [b5]"},
  {"C4","Db4","Eb4","F4","G4","Ab4","Bb4","C5","Db5","Eb5","F5","G5","Ab5","Bb5","C6","Db6","-","Ab/Fm [b4]"},
  {"C4","D4","Eb4","F4","G4","Ab4","Bb4","C5","D5","Eb5","F5","G5","Ab5","Bb5","C6","D6","-","Eb/Cm [b3]"},
  {"C4","D4","Eb4","F4","G4","A4","Bb4","C5","D5","Eb5","F5","G5","A5","Bb5","C6","D6","-","Bb/Gm [b2]"},
  {"C4","D4","E4","F4","G4","A4","Bb4","C5","D5","E5","F5","G5","A5","Bb5","C6","D6","-","F/Dm [b1]"},
};

SSD1306 display(0x3c, 5, 4); // /SSD1306インスタンスの作成（I2Cアドレス,SDA,SCL）
WiFiServer server(80);
WiFiUDP udp;
// called this way, it uses the default address 0x40
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

String lip;
String msg="SetTone, Press =";
int nk=0;
int count=0;

void drawLcd(int n){
  display.init();    //ディスプレイを初期化
  display.setFont(ArialMT_Plain_16);    //フォントを設定
  display.drawString(0, 0, lip);
  display.drawString(0, 20,"("+String(n+1)+") "+labels[n][17]);
  display.drawString(0, 40,msg);
  display.display();   //指定された情報を描画
}

void playM(int ch){
  if (ch>=0 && ch<16){
    msg=labels[nk][ch];
    drawLcd(nk);
    ledcWriteTone(1,tones[nk][ch]);
    ledWrite(ch,BRIGHT);
    delay(BEAT);
    ledcWriteTone(1,0);
    ledWrite(ch,0);
  }
}

void playA(int ch,int dur){
  if (ch>=0 && ch<16){
    count++;
    msg=String(count)+"--> "+labels[nk][ch];
    drawLcd(nk);
    ledcWriteTone(1,tones[nk][ch]);
    ledWrite(ch,BRIGHT);
    delay(dur*TEMPO);
    ledcWriteTone(1,0);
    ledWrite(ch,0);
  }
}

void pause(int dur){
  count++;
  msg=String(count)+"--> PASUE";
  drawLcd(nk);
  delay(dur*TEMPO);
}

void setup() {
  WiFi.mode(WIFI_STA);                    // 無線LANをSTAモードに設定
  WiFi.begin(SSID,PASS);                  // 無線LANアクセスポイントへ接続
  while(WiFi.status() != WL_CONNECTED){   // 接続に成功するまで待つ
    delay(500);                           // 待ち時間処理
  }
  
  lip=WiFi.localIP().toString();   // IPアドレスを取得
  // 調を取得
  udp.begin(R_PORT);
  int chk=0;
  drawLcd(nk);
  while (chk==0){
    char lcdp[20];                          // 表示用変数を定義(49バイト48文字)
    int len;                                // 文字列長を示す整数型変数を定義
    memset(lcdp, 0, 20);                    // 文字列変数lcdの初期化(49バイト)
    len = udp.parsePacket();                // 受信パケット長を変数lenに代入
    if(len>0){ 
      udp.read(lcdp, 20);                   // 受信データを文字列変数lcdpへ代入
      if (lcdp[0]=='0') nk--;
      if (nk<0) nk=11;
      if (lcdp[0]=='`') nk++;
      if (nk>11) nk=0;
      if (lcdp[0]=='=') chk=1;
      drawLcd(nk);
    }
  }
  msg="Start";
  drawLcd(nk);
  pwm.begin();
  pwm.setPWMFreq(1600); // This is the maximum PWM frequency
  ledcSetup(1,12000, 8);
  ledcAttachPin(BUZZER_PIN,1);
  for( int i = 0; i <16; i++ ) {
    ledWrite(i,0);
  }
}
void loop() {
  char lcdp[20];                          
  int len;                                
  memset(lcdp, 0, 20);                    
  len = udp.parsePacket();                
  if(len>0){
    udp.read(lcdp, 20);
    // 受信データを文字列変数lcdpへ代入
    char c=lcdp[0];
    if (c=='.') {
      msg="Demo";
      playDemo();
    } else if (c=='0') {
      nk--;
      if (nk<0) nk=11;
      msg="-";
      drawLcd(nk);
    } else if (c=='`'){
      nk++;
      if (nk>11) nk=0;
      msg="+";
      drawLcd(nk);
    } else {
      for (int i=0; i<16; i++){
        char buf[2];
        String k=keys[i];
        k.toCharArray(buf,2);
        if (c==buf[0]){
          msg=labels[nk][i];
          playM(i);
          break;
        }
      }
    }
  }
}

void ledWrite( int n, int power ) {
  power = map( power, 0, 255, PULSE_MIN, PULSE_MAX );  // powerを256段階に変換
  pwm.setPWM( n, 0, power );  // LED 点灯
}

void playDemo(){
  nk=2;
  count=0;
  playA(8,4); //1
  playA(10,4); //2
  playA(12,4); //3
  playA(11,4); //4

  playA(10,4); //5
  playA(9,4); //6
  playA(11,4); //7
  playA(10,4); //8

  playA(8,4); //9
  playA(10,4); //10
  playA(9,6); //11
  playA(8,2); //12

  playA(8,12); //13
  pause(4); //14

  playA(5,2); //15
  playA(5,2); //16
  playA(5,2); //17
  playA(5,2); //18
  playA(5,2); //19
  playA(5,2); //20
  playA(5,2); //21
  playA(5,2); //22

  playA(6,2); //23
  playA(7,2); //24
  playA(8,2); //25
  playA(9,2); //26
  playA(7,4); //27
  pause(4); //28

  playA(6,2); //29
  playA(8,2); //30
  playA(11,2); //31
  playA(10,4); //32
  playA(5,2); //33
  playA(9,2); //34
  playA(8,2); //35

  playA(8,6); //36
  playA(7,2); //37
  playA(7,4); //38
  pause(4); //39

  playA(5,2); //40
  playA(5,2); //41
  playA(5,2); //42
  playA(5,2); //43
  playA(5,2); //44
  playA(5,2); //45
  playA(5,2); //46
  playA(5,2); //47

  playA(6,2); //48
  playA(7,2); //49
  playA(8,2); //50
  playA(9,2); //51
  playA(7,4); //52
  pause(4); //53

  playA(6,2); //54
  playA(8,2); //55
  playA(11,2); //56
  playA(10,4); //57
  playA(5,2); //58
  playA(9,2); //59
  playA(8,2); //60

  playA(8,12); //61
  pause(4); //62

  playA(6,4); //63
  playA(8,4); //64
  playA(9,4); //65
  playA(7,1); //66
  playA(6,1); //67
  playA(5,2); //68

  playA(6,4); //69
  playA(6,2); //70
  playA(8,2); //71
  playA(9,4); //72
  pause(2); //73
  playA(10,2); //74

  playA(10,4); //75
  playA(9,2); //76
  playA(8,2); //77
  playA(9,4); //78
  pause(2); //79
  playA(10,2); //80

  playA(11,4); //81
  playA(10,2); //82
  playA(9,2); //83
  playA(10,4); //84
  pause(4); //85

  playA(6,4); //86
  playA(8,4); //87
  playA(9,4); //88
  playA(7,1); //89
  playA(6,1); //90
  playA(5,2); //91

  playA(6,4); //92
  playA(6,2); //93
  playA(8,2); //94
  playA(9,4); //95
  pause(2); //96
  playA(10,2); //97

  playA(11,4); //98
  playA(10,4); //99
  playA(9,4); //100
  playA(8,4); //101

  playA(7,2); //102
  pause(2); //103
  playA(7,4); //104
  playA(8,4); //105
  playA(9,4); //106

  playA(8,4); //107
  playA(10,4); //108
  playA(12,4); //109
  playA(11,4); //110

  playA(10,4); //111
  playA(9,4); //112
  playA(11,4); //113
  playA(10,4); //114

  playA(8,4); //115
  playA(10,4); //116
  playA(9,6); //117
  playA(8,2); //118

  playA(8,12); //119
  pause(4); //120

  playA(8,4); //121
  playA(10,4); //122
  playA(12,4); //123
  playA(11,4); //124

  playA(10,4); //125
  playA(9,4); //126
  playA(11,4); //127
  playA(10,4); //128

  playA(8,4); //129
  playA(10,4); //130
  playA(9,6); //131
  playA(8,2); //132

  playA(8,12); //133
  pause(4); //134
}
